//******************************************************************************
//* TDateTimePickerLink                                                        *
//*                                                                            *
//* version:     1.1                                                           *
//* date:        12.08.2000                                                    *
//* compiler:    Delphi 5.0                                                    *
//* author:      Inger Hoffmann copyright  2000                               *
//* e-mail:      ingerhoffmann@web.de                                          *
//*                                                                            *
//******************************************************************************
//* important note:                                                            *
//* The source code is given as is. The author is not responsible              *
//* for any possible damage done due to the use of this code.                  *
//* The component can be freely used in any application. The source            *
//* code remains property of the author and may not be distributed             *
//* freely as such nor sold in any form.                                       *
//*                                                                            *
//******************************************************************************
//* description:                                                               *
//* version 1.0                                                                *
//* TDateTimePickerLink is a non visual component, that is linking two         *
//* DateTimePicker controls.                                                   *
//* If the value of the first DateTimePicker control is higher than the value  *
//* of the second DateTimePicker control a message is shown and the changed    *
//* value will be set to the value of the other DateTimePicker control.        *
//* The message text is a published property.                                  *
//* version 1.1                                                                *
//* With the property MaxFromToValue it is possible to set a maximum value for *
//* the period of time. For a date the unit is 'day' and for time the unit is  *
//* 'minute'. The message text displayed when exceeding the period is the      *
//* published property MaxMessageText.                                         *
//******************************************************************************
unit DateTimePickerLink;

interface

uses
  Classes, Dialogs, Comctrls, Sysutils;

type
  TDateTimePickerLink = class(TComponent)
  private
    FFrom : TDateTimePicker;
    FTo : TDateTimePicker;
    FMaxFromToValue : integer;
    FMaxFromToValueHour : integer;
    FMaxFromToValueMin : integer;
    FMessage : string;
    FMaxMessage : string;
    procedure SetFrom(FDateTimePicker : TDateTimePicker);
    procedure SetTo(FDateTimePicker : TDateTimePicker);
    procedure SetMaxFromToValue(iValue : integer);
    procedure SetMessage(FString : string);
    procedure SetMaxMessage(FString : string);
    procedure CheckFrom(Sender: TObject);
    procedure CheckTo(Sender: TObject);
    function MaxFromToValueExceeded : boolean;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    property DateFrom : TDateTimePicker read FFrom write SetFrom;
    property DateTo : TDateTimePicker read FTo write SetTo;
    property MaxFromToValue : integer read FMaxFromToValue write SetMaxFromToValue;
    property MaxMessageText : string read FMaxMessage write SetMaxMessage;
    property MessageText : string read FMessage write SetMessage;
  end;

procedure Register;

implementation

//******************************************************************************
constructor TDateTimePickerLink.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMaxFromToValueHour := 0;
  FMaxFromToValueMin := 0;
end;

//******************************************************************************
destructor TDateTimePickerLink.Destroy;
begin
  inherited destroy;
end;

//******************************************************************************
procedure TDateTimePickerLink.SetFrom(FDateTimePicker : TDateTimePicker);
begin
  FFrom := FDateTimePicker;
  FFrom.OnCloseUp := CheckFrom;
  FFrom.OnExit := CheckFrom;
end;

//******************************************************************************
procedure TDateTimePickerLink.SetTo(FDateTimePicker : TDateTimePicker);
begin
  FTo := FDateTimePicker;
  FTo.OnCloseUp := CheckTo;
  FTo.OnExit := CheckTo;
end;

//******************************************************************************
procedure TDateTimePickerLink.SetMessage(FString : string);
begin
  FMessage := FString;
end;

//******************************************************************************
procedure TDateTimePickerLink.SetMaxMessage(FString : string);
begin
  FMaxMessage := FString;
end;

//******************************************************************************
procedure TDateTimePickerLink.SetMaxFromToValue(iValue : integer);
begin
  FMaxFromToValue := iValue;
  FMaxFromToValueMin := FMaxFromToValue mod 60;
  FMaxFromToValueHour := round((FMaxFromToValue - FMaxFromToValueMin)/60);
end;

//******************************************************************************
procedure TDateTimePickerLink.CheckFrom;
var
  HourTo, MinTo, SecTo, MSecTo: Word;
begin
  if not assigned (FFrom) then exit;
  if not assigned (FTo) then exit;
  if (FFrom.Kind = dtkDate) and (FTo.Kind = dtkDate) then
  begin
    if FFrom.Date > FTo.Date then
    begin
      FFrom.Date := FTo.Date;
      MessageDlg(FMessage,mtInformation,[mbOk],0);
    end
    else if (FTo.Date - FFrom.Date > FMaxFromToValue) and (FMaxFromToValue > 0) then
    begin
      FFrom.Date := FTo.Date - FMaxFromToValue;
      MessageDlg(FMaxMessage,mtInformation,[mbOk],0);
    end;
  end
  else if (FFrom.Kind = dtkTime) and (FTo.Kind = dtkTime) then
  begin
    if FFrom.Time > FTo.Time then
    begin
      FFrom.Time := FTo.Time;
      MessageDlg(FMessage,mtInformation,[mbOk],0);
    end
    else if (FMaxFromToValue > 0) then
    begin
      DecodeTime(FTo.Time,HourTo,MinTo,SecTo,MSecTo);
      if MaxFromToValueExceeded then
      begin
        FFrom.Time := EncodeTime(HourTo - FMaxFromToValueHour,MinTo - FMaxFromToValueMin,SecTo,MSecTo);
        MessageDlg(FMaxMessage,mtInformation,[mbOk],0);
      end;
    end;
  end;
end;

//******************************************************************************
procedure TDateTimePickerLink.CheckTo;
var
  HourFrom, MinFrom, SecFrom, MSecFrom: Word;
begin
  if not assigned (FFrom) then exit;
  if not assigned (FTo) then exit;
  if (FFrom.Kind = dtkDate) and (FTo.Kind = dtkDate) then
  begin
    if FFrom.Date > FTo.Date then
    begin
      FTo.Date := FFrom.Date;
      MessageDlg(FMessage,mtInformation,[mbOk],0);
    end
    else if (FTo.Date - FFrom.Date > FMaxFromToValue) and (FMaxFromToValue > 0) then
    begin
      FTo.Date := FFrom.Date + FMaxFromToValue;
      MessageDlg(FMaxMessage,mtInformation,[mbOk],0);
    end;
  end
  else if (FFrom.Kind = dtkTime) and (FTo.Kind = dtkTime) then
  begin
    if FFrom.Time > FTo.Time then
    begin
      FTo.Time := FFrom.Time;
      MessageDlg(FMessage,mtInformation,[mbOk],0);
    end
    else if (FMaxFromToValue > 0) then
    begin
      DecodeTime(FFrom.Time,HourFrom,MinFrom,SecFrom,MSecFrom);
      if MaxFromToValueExceeded then
      begin
        FTo.Time := EncodeTime(HourFrom + FMaxFromToValueHour,MinFrom + FMaxFromToValueMin,SecFrom,MSecFrom);
        MessageDlg(FMaxMessage,mtInformation,[mbOk],0);
      end;
    end;
  end;
end;

//******************************************************************************
function TDateTimePickerLink.MaxFromToValueExceeded : boolean;
var
  HourFrom, MinFrom, SecFrom, MSecFrom: Word;
  HourTo, MinTo, SecTo, MSecTo: Word;
begin
  MaxFromToValueExceeded := false;
  DecodeTime(FFrom.Time,HourFrom,MinFrom,SecFrom,MSecFrom);
  DecodeTime(FTo.Time,HourTo,MinTo,SecTo,MSecTo);
  if (((HourTo * 60 + MinTo) - (HourFrom * 60 + MinFrom)) > FMaxFromToValue)
    or ((((HourTo * 60 + MinTo) - (HourFrom * 60 + MinFrom)) = FMaxFromToValue)
    and ((SecTo > SecFrom) or (MSecTo > MSecFrom))) then
      MaxFromToValueExceeded := true;
end;

//******************************************************************************
procedure Register;
begin
  RegisterComponents('External', [TDateTimePickerLink]);
end;

//******************************************************************************
end.




